@@ -2,6 +2,7 @@ import pytest
 import pandas as pd
 import numpy as np
 import pandas.util.testing as tm
+
 pytest.importorskip('rbc')
 
 
@@ -10,19 +11,21 @@ def catch_udf_support_disabled(mth):
         try:
             return mth(self, con)
         except Exception as msg:
-            if (type(msg).__name__ == 'TOmniSciException'
-                and msg.error_msg.startswith(
-                    'Runtime UDF registration is disabled')):
+            if type(
+                msg
+            ).__name__ == 'TOmniSciException' and msg.error_msg.startswith(
+                'Runtime UDF registration is disabled'
+            ):
                 print('Ignoring `%s` failure' % (msg.error_msg))
                 return
             raise
+
     new_mth.__name__ = mth.__name__
     return new_mth
 
 
 @pytest.mark.usefixtures("mapd_server")
 class TestRuntimeUDF:
-
     def load_test_udf_incr(self, con):
         con.execute('drop table if exists test_udf_incr')
         con.execute('create table test_udf_incr (i4 integer, f8 double)')
@@ -31,7 +34,6 @@ class TestRuntimeUDF:
 
     @catch_udf_support_disabled
     def test_udf_incr(self, con):
-
         @con('int32(int32)', 'double(double)')
         def incr(x):
             return x + 1
@@ -50,7 +52,6 @@ class TestRuntimeUDF:
 
     @catch_udf_support_disabled
     def test_udf_incr_read_sql(self, con):
-
         @con('int32(int32)', 'double(double)')
         def incr_read_sql(x):
             return x + 1
@@ -59,20 +60,28 @@ class TestRuntimeUDF:
 
         result = pd.read_sql(
             '''select i4 as qty, incr_read_sql(i4) as price
-               from test_udf_incr''', con)
-        expected = pd.DataFrame({
-            "qty": np.array([1, 2], dtype=np.int64),
-            "price": np.array([2, 3], dtype=np.int64)
-        })[['qty', 'price']]
+               from test_udf_incr''',
+            con,
+        )
+        expected = pd.DataFrame(
+            {
+                "qty": np.array([1, 2], dtype=np.int64),
+                "price": np.array([2, 3], dtype=np.int64),
+            }
+        )[['qty', 'price']]
         tm.assert_frame_equal(result, expected)
 
         result = pd.read_sql(
             '''select f8 as qty, incr_read_sql(f8) as price
-               from test_udf_incr''', con)
-        expected = pd.DataFrame({
-            "qty": np.array([2.3, 3.4], dtype=np.float64),
-            "price": np.array([3.3, 4.4], dtype=np.float64)
-        })[['qty', 'price']]
+               from test_udf_incr''',
+            con,
+        )
+        expected = pd.DataFrame(
+            {
+                "qty": np.array([2.3, 3.4], dtype=np.float64),
+                "price": np.array([3.3, 4.4], dtype=np.float64),
+            }
+        )[['qty', 'price']]
         tm.assert_frame_equal(result, expected)
 
         con.execute('drop table if exists test_udf_incr')
